@@ -6,10 +6,9 @@
 ;; COPYRIGHT   : (C) 2001  Joris van der Hoeven
 ;;               (C) 2004, 2005, 2006  Joris van der Hoeven and Henri Lesourd
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -57,8 +56,6 @@
 ;;
 ;;   -> On en discutera davantage apres un premier passage en revue.
 
-;; FIXME: Les cercles et arcs de cercles sont bogues
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Edit mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -80,6 +77,9 @@
 (define (object-set! o . opt)
  ;(display* "o=" o "\n")
   (set! layer-of-last-removed-object #f)
+  (set! current-obj o)    ;; FIXME: Hmmm, I still have a doubt for this one.
+                          ;;   Should completely clarify its role & centralize
+                          ;;   where it's managed (and for similar gvs as well).
   (if sticky-point
       (sketch-set! `(,o))
       (if (in? 'checkout opt)
@@ -88,9 +88,7 @@
 	     (sketch-checkout))
 	  (if (in? 'new opt)
 	      (graphics-group-enrich-insert o)
-	      (graphics-assign current-path o)))
-  )
-  (set! current-obj o))
+	      (graphics-assign current-path o)))))
 
 ;; Basic operations (create)
 (define (sketch-get1)
@@ -106,9 +104,8 @@
   (:require (in? tag gr-tags-curves))
   (with o (graphics-enrich `(,tag (point ,x ,y) (point ,x ,y)))
     (graphics-store-state 'start-create)
-    (object-set! o 'checkout)
-    (set! current-obj o)
     (set! current-point-no 1)
+    (object-set! o 'checkout)
     (graphics-store-state #f)))
 
 (tm-define (object_create tag x y)
@@ -118,14 +115,13 @@
 ;; Basic operations (set & add point)
 (define (object_set-point no xcur ycur)
   (define obj (stree-radical (car (sketch-get1))))
- ;(display* "obj=" obj "\n")
+ ;(display* "obj[" no ";" xcur "," ycur "]=" obj "\n")
   (if (== (car obj) 'point)
       (begin
 	 (set-car! (cdr obj) xcur)
 	 (set-car! (cddr obj) ycur))
-      (if (not (and (in? (car obj) '(arc carc)) (> (length obj) 3)))
-          (with l (list-tail (cdr obj) no)
-	    (set-car! l `(point ,xcur ,ycur)))))
+      (with l (list-tail (cdr obj) no)
+	 (set-car! l `(point ,xcur ,ycur))))
   (object-set! (car (sketch-get))))
 
 (define (object_add-point no xcur ycur x y dirn)
@@ -281,7 +277,11 @@
 		  previous-leftclick
                  `(point ,current-x ,current-y)
 		  moveclick-tolerance))
-            (undo)
+	    (begin
+	       (undo)
+	       (set! choosing #f)
+	       (set! leftclick-waiting #f)
+	       (set! just-started-dragging #f))
 	    (begin
 	      (set-message "Left click: finish" "")
 	      (set! leftclick-waiting #t)))))
@@ -291,7 +291,6 @@
   (set! previous-leftclick `(point ,current-x ,current-y)))
 
 (define (back)
-;; FIXME: Doesn't undo the right point, it seems
  ;(display* "obj[" p "]=" obj "\n")
   (graphics-back-state #f)
   (graphics-move-point current-x current-y))
@@ -387,13 +386,17 @@
 	(left-button))
       (if current-obj
 	  (begin
-	    (edit_tab-key 'edit
-			  (if (graphics-states-void?)
-			      #f
-			      choosing))
-	    (if (not (graphics-states-void?))
-		(graphics-pop-state))
-	    (graphics-store-state #f)
+	    (if (not just-started-dragging)
+	    (begin
+	       (edit_tab-key 'edit
+			     (if (graphics-states-void?)
+				 #f
+				 choosing))))
+	    (if (or (not just-started-dragging) (not choosing))
+	    (begin
+	       (if (not (graphics-states-void?))
+		   (graphics-pop-state))
+	       (graphics-store-state #f)))
 	    (set! choosing #t))
 	  (edit-insert x y))))
 
@@ -869,7 +872,8 @@
       )
       (begin
 	 (if sticky-point (undo))
-	 (graphics-decorations-update))))
+	 (sketch-reset)
+	 (graphics-decorations-reset))))
 
 (tm-define (graphics-finish)
   ;;(display* "Graphics] Finish\n")
