@@ -4,10 +4,9 @@
 * DESCRIPTION: Mouse handling
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "edit_interface.hpp"
@@ -57,7 +56,7 @@ edit_interface_rep::mouse_any (string type, SI x, SI y, int mods, time_t t) {
       dragging= true;
     }
     else if (dragging && (type == "move"))
-      type2= "dragging"; 
+      type2= "dragging";
     if (dragging && (type == "release-left"))
       type2= "end-drag";
 
@@ -67,7 +66,7 @@ edit_interface_rep::mouse_any (string type, SI x, SI y, int mods, time_t t) {
       right_dragging= true;
     }
     else if (right_dragging && (type == "move"))
-      type2= "right-dragging"; 
+      type2= "right-dragging";
     if (right_dragging && (type == "release-right"))
       type2= "end-right-drag";
 
@@ -88,10 +87,10 @@ edit_interface_rep::mouse_any (string type, SI x, SI y, int mods, time_t t) {
   if (type == "release-left" || type == "release-right") {
     dragging= right_dragging= false;
     send_mouse_grab (this, false);
-    if ((t >= last_click) && ((t - last_click) <= 250)) {
+    if ((t >= last_click) && ((t - last_click) <= 500)) {
       last_click= t;
       if (mouse_extra_click (x, y))
-	last_click= t- 1000;
+	last_click= t- 2000;
     }
     else {
       last_click= t;
@@ -133,6 +132,8 @@ edit_interface_rep::mouse_extra_click (SI x, SI y) {
   get_selection (p1, p2);
   if ((p1==p2) || path_less (tp, p1) || path_less (p2, tp)) select (tp, tp);
   select_enlarge ();
+  if (selection_active_any ())
+    selection_set ("mouse", selection_get (), true);
   return false;
 }
 
@@ -140,6 +141,7 @@ void
 edit_interface_rep::mouse_drag (SI x, SI y) {
   if (inside_graphics ()) return;
   if (eb->action ("drag", x, y, 0) != "") return;
+  go_to (x, y);
   end_x  = x;
   end_y  = y;
   selection_visible ();
@@ -151,8 +153,8 @@ edit_interface_rep::mouse_drag (SI x, SI y) {
     p1= p2;
     p2= temp;
   }
-  set_selection (p1, p2);
   if ((p1 == p2) && start_drag) return;
+  set_selection (p1, p2);
   start_drag= start_right_drag= false;
   notify_change (THE_SELECTION);
 }
@@ -180,15 +182,21 @@ edit_interface_rep::mouse_select (SI x, SI y, int mods) {
     invalidate_graphical_object ();
     eval ("(graphics-reset-context 'exit)");
   }
+  if (start_drag) {
+    path sp= find_innermost_scroll (eb, tp);
+    path p0= tree_path (sp, x, y, 0);
+    set_selection (p0, p0);
+    notify_change (THE_SELECTION);
+  }
   if (selection_active_any ())
-    selection_set ("primary", selection_get (), true);
+    selection_set ("mouse", selection_get (), true);
 }
 
 void
 edit_interface_rep::mouse_paste (SI x, SI y) { (void) x; (void) y;
   if (eb->action ("paste", x, y, 0) != "") return;
-  selection_copy ();
-  selection_paste ();
+  go_to (x, y);
+  selection_paste ("mouse");
 }
 
 void
