@@ -4,10 +4,9 @@
 * DESCRIPTION: Renderer for printing post-script graphics
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "printer.hpp"
@@ -307,7 +306,7 @@ printer_rep::select_tex_font (string name) {
 * make tex characters and fonts
 ******************************************************************************/
 
-static char* hex_string= "0123456789ABCDEF";
+static const char* hex_string= "0123456789ABCDEF";
 
 void
 printer_rep::make_tex_char (string name, unsigned char c, glyph gl) {
@@ -511,7 +510,7 @@ printer_rep::set_background (color c) {
 void
 printer_rep::draw (int ch, font_glyphs fn, SI x, SI y) {
   glyph gl= fn->get(ch);
-  if (nil (gl)) return;
+  if (is_nil (gl)) return;
   string name= fn->res_name;
   unsigned char c= ch;
   if (ch >= 256) {
@@ -609,7 +608,7 @@ printer_rep::polygon (array<SI> x, array<SI> y, bool convex) {
 void
 printer_rep::xpm (url file_name, SI x, SI y) {
   (void) file_name; (void) x; (void) y;
-  fatal_error ("Not yet implemented", "printer_rep::xpm");
+  FAILED ("not yet implemented");
 }
 
 /*
@@ -755,6 +754,6 @@ renderer
 printer (url ps_file_name, int dpi, int nr_pages,
 	 string page_type, bool landscape, double paper_w, double paper_h)
 {
-  return new printer_rep (ps_file_name, dpi, nr_pages,
+  return tm_new<printer_rep> (ps_file_name, dpi, nr_pages,
 			  page_type, landscape, paper_w, paper_h);
 }
