@@ -4,10 +4,9 @@
 * DESCRIPTION: fixed size trees with reference counting
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef TREE_H
@@ -23,6 +22,8 @@
 class tree_rep;
 class atomic_rep;
 class compound_rep;
+tree copy (tree t);
+
 class tree {
   tree_rep* rep; // can be atomic or compound
   inline tree (tree_rep* rep2);
@@ -62,6 +63,7 @@ public:
   friend inline bool operator != (tree t, string s);
   friend inline bool operator == (tree t, const char* s);
   friend inline bool operator != (tree t, const char* s);
+  friend inline tree_rep* inside (tree t);
   friend inline bool strong_equal (tree t, tree u);
   friend inline bool is_func (tree t, tree_label l);
   friend inline bool is_func (tree t, tree_label l, int i);
@@ -78,6 +80,8 @@ public:
   friend list<tree> as_trees (list<pointer> l);
   friend class tree_pointer_rep;
   friend class tree_position_rep;
+  friend class edit_observer_rep;
+  friend class undo_observer_rep;
   friend class tree_links_rep;
   friend class link_repository_rep;
 };
@@ -113,19 +117,19 @@ typedef tree scheme_tree;
 ******************************************************************************/
 
 #ifdef debug_trees
-#define CHECK_ATOMIC(t,s) \
+#define CHECK_ATOMIC(t) \
   if (((t).rep)->op != STRING) { \
     cerr << "\nThe tree : " << (t) << "\n"; \
-    fatal_error ("atomic tree expected", s); \
+    FAILED ("atomic tree expected"); \
   }
-#define CHECK_COMPOUND(t,s) \
+#define CHECK_COMPOUND(t) \
   if (((t).rep)->op == STRING) { \
     cerr << "\nThe tree : " << (t) << "\n"; \
-    fatal_error ("compound tree expected", s); \
+    FAILED ("compound tree expected"); \
   }
 #else
-#define CHECK_ATOMIC(t,s)
-#define CHECK_COMPOUND(t,s)
+#define CHECK_ATOMIC(t)
+#define CHECK_COMPOUND(t)
 #endif
 
 void destroy_tree_rep (tree_rep* rep);
@@ -134,7 +138,7 @@ inline tree::tree (const tree& x): rep (x.rep) { rep->ref_count++; }
 inline tree::~tree () {
   if ((--rep->ref_count)==0) { destroy_tree_rep (rep); rep= NULL; } }
 inline atomic_rep* tree::operator -> () {
-  CHECK_ATOMIC (*this, "tree::operator ->");
+  CHECK_ATOMIC (*this);
   return static_cast<atomic_rep*> (rep); }
 inline tree& tree::operator = (tree x) {
   x.rep->ref_count++;
@@ -143,24 +147,24 @@ inline tree& tree::operator = (tree x) {
   return *this; }
 
 inline tree::tree ():
-  rep (new atomic_rep (string ())) {}
+  rep (tm_new<atomic_rep> (string ())) {}
 inline tree::tree (const char *s):
-  rep (new atomic_rep (s)) {}
+  rep (tm_new<atomic_rep> (s)) {}
 inline tree::tree (string s):
-  rep (new atomic_rep (s)) {}
+  rep (tm_new<atomic_rep> (s)) {}
 inline tree::tree (tree_label l, int n):
-  rep (new compound_rep (l, array<tree> (n))) {}
+  rep (tm_new<compound_rep> (l, array<tree> (n))) {}
 inline tree::tree (tree_label l, array<tree> a):
-  rep (new compound_rep (l, a)) {}
+  rep (tm_new<compound_rep> (l, a)) {}
 inline tree::tree (tree t, int n):
-  rep (new compound_rep (t.rep->op, array<tree> (n))) {
-    CHECK_COMPOUND (t, "tree::tree (tree, int)"); }
+  rep (tm_new<compound_rep> (t.rep->op, array<tree> (n))) {
+    CHECK_COMPOUND (t); }
 
 inline tree& tree::operator [] (int i) {
-  CHECK_COMPOUND (*this, "tree::operator []");
+  CHECK_COMPOUND (*this);
   return (static_cast<compound_rep*> (rep))->a[i]; }
 inline int N (tree t) {
-  CHECK_COMPOUND (t, "N (tree)");
+  CHECK_COMPOUND (t);
   return N ((static_cast<compound_rep*> (t.rep))->a); }
 inline int arity (tree t) {
   if (t.rep->op == STRING) return 0;
@@ -172,10 +176,10 @@ inline tree_label L (tree t) {
 inline tree_label& LR (tree t) {
   return t.rep->op; }
 inline array<tree> A (tree t) {
-  CHECK_COMPOUND (t, "A (tree)");
+  CHECK_COMPOUND (t);
   return (static_cast<compound_rep*> (t.rep))->a; }
 inline array<tree>& AR (tree t) {
-  CHECK_COMPOUND (t, "AR (tree)");
+  CHECK_COMPOUND (t);
   return (static_cast<compound_rep*> (t.rep))->a; }
 
 inline bool is_atomic (tree t) { return (t.rep->op == STRING); }
@@ -194,8 +198,10 @@ inline bool operator == (tree t, const char* s) {
   return (t.rep->op == STRING) && (t->label == s); }
 inline bool operator != (tree t, const char* s) {
   return (t.rep->op != STRING) || (t->label != s); }
+inline tree_rep* inside (tree t) {
+  return t.rep; }
 inline bool strong_equal (tree t, tree u) {
-  return t.rep == u. rep; }
+  return t.rep == u.rep; }
 
 inline bool is_func (tree t, tree_label l) {
   return (t.rep->op==l) && (N(t)!=0); }
