@@ -4,10 +4,9 @@
 * DESCRIPTION: Graphical user interface for X11
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "timer.hpp"
@@ -30,7 +29,7 @@ x_character_rep::x_character_rep (
     c (c2), fng (fng2), sf (sf2), fg (fg2), bg (bg2) {}
 
 x_character::x_character (int c, font_glyphs fng, int sf, color fg, color bg):
-  rep (new x_character_rep (c, fng, sf, fg, bg)) {}
+  rep (tm_new<x_character_rep> (c, fng, sf, fg, bg)) {}
 
 x_character::operator tree () {
   tree t (TUPLE,  as_string (rep->c), rep->fng->res_name);
@@ -60,7 +59,7 @@ x_gui_rep::prepare_color (int sf, color fg, color bg) {
     int fR, fG, fB, bR, bG, bB, j;
     get_rgb_color (fg, fR, fG, fB);
     get_rgb_color (bg, bR, bG, bB);
-    cols= new color [nr_cols+1];
+    cols= tm_new_array<color> (nr_cols+1);
     for (j=0; j<=nr_cols; j++)
       cols [nr_cols-j]= rgb_color ((bR*j + fR*(nr_cols-j)) / nr_cols,
 				   (bG*j + fG*(nr_cols-j)) / nr_cols,
@@ -189,7 +188,7 @@ x_gui_rep::has_mouse_grab (widget w) {
 ******************************************************************************/
 
 Bool
-my_predicate (Display* dpy, XEvent* ev, XPointer arg) { (void) dpy;
+my_selnotify_predicate (Display* dpy, XEvent* ev, XPointer arg) { (void) dpy;
   x_window win= (x_window) arg;
   return (win->win==ev->xany.window) && (ev->type==SelectionNotify);
 }
@@ -213,29 +212,38 @@ bool
 x_gui_rep::get_selection (string key, tree& t, string& s) {
   t= "none";
   s= "";
+  bool res=false;
+
   if (selection_t->contains (key)) {
     t= copy (selection_t [key]);
     s= copy (selection_s [key]);
-    return true;
+    res=true;
   }
-  if (key != "primary") return false;
-  if (XGetSelectionOwner (dpy, XA_PRIMARY) == None) return false;
   
-  if (is_nil (windows_l)) return false;
+  Atom xsel;
+  if (key == "primary") xsel = XA_CLIPBOARD;
+  else if (key == "mouse") xsel = XA_PRIMARY;
+  else return res;
+
+  Window selown = XGetSelectionOwner(dpy, xsel);
+  if (selown == None
+  || is_nil (windows_l)
+  || contains(windows_l,selown)) return res;
+
   Window win= windows_l->item;
   x_window x_win= (x_window) Window_to_window[win];
-  Atom data= XInternAtom (dpy, "MY_STRING_SELECTION", false);
-  XConvertSelection (dpy, XA_PRIMARY, XA_STRING, data, win, CurrentTime);
+  Atom prop= XInternAtom (dpy, "MY_STRING_SELECTION", false);
+  XConvertSelection (dpy, xsel, XA_STRING, prop, win, CurrentTime);
 
   int i;
   XEvent ev;
   for (i=0; i<1000000; i++)
-    if (XCheckIfEvent (dpy, &ev, my_predicate, (XPointer) x_win))
+    if (XCheckIfEvent (dpy, &ev, my_selnotify_predicate, (XPointer) x_win))
       break;
-  if (i==1000000) return false;
+  if (i==1000000) return res;
   XSelectionEvent& sel= ev.xselection;
 
-  if (sel.property!=None) {
+  if (sel.property==prop) {
     long offset=0;
     Atom type;
     int fm;
@@ -250,23 +258,24 @@ x_gui_rep::get_selection (string key, tree& t, string& s) {
       offset += (n >> 2);
       XFree (ret);
     } while (remains>0);
-  }
-  t= tuple ("extern", s);
-  return true;
+    t= tuple ("extern", s);
+    return true;
+  } else
+    return res;
 }
 
 bool
 x_gui_rep::set_selection (string key, tree t, string s) {
   selection_t (key)= copy (t);
   selection_s (key)= copy (s);
-  if (key == "primary") {
-    if (is_nil (windows_l)) return false;
-    Window win= windows_l->item;
-    if (selection!=NULL) delete[] selection;
-    XSetSelectionOwner (dpy, XA_PRIMARY, win, CurrentTime);
-    if (XGetSelectionOwner(dpy, XA_PRIMARY)==None) return false;
-    selection= as_charp (s);
-  }
+  Atom xsel;
+  if(key == "primary") xsel = XA_CLIPBOARD;
+  else if (key == "mouse") xsel = XA_PRIMARY;
+  else return true;
+  if (is_nil (windows_l)) return false;
+  Window win= windows_l->item;
+  XSetSelectionOwner (dpy, xsel, win, CurrentTime);
+  if (XGetSelectionOwner(dpy, xsel)==None) return false;
   return true;
 }
 
@@ -274,10 +283,6 @@ void
 x_gui_rep::clear_selection (string key) {
   selection_t->reset (key);
   selection_s->reset (key);
-  if ((key == "primary") && (selection != NULL)) {
-    delete[] selection;
-    selection= NULL;
-  }
 }
 
 bool
@@ -440,10 +445,10 @@ void
 x_gui_rep::set_mouse_pointer (widget w, string name, string mask_name) {
   static hashmap<string,tree> xpm_cache ("");
   if (mask_name=="") mask_name= name;
-  x_drawable_rep* dra= new x_drawable_rep (this);
+  x_drawable_rep* dra= tm_new<x_drawable_rep> (this, 1, 1);
   dra->xpm_initialize (name);
   if (mask_name!=name) dra->xpm_initialize (mask_name);
-  delete dra;
+  tm_delete (dra);
   Pixmap curs= (Pixmap) xpm_bitmap [name];
   Pixmap mask= (Pixmap) xpm_bitmap [mask_name];
 
@@ -455,8 +460,7 @@ x_gui_rep::set_mouse_pointer (widget w, string name, string mask_name) {
 
   array<string> cnames_curs= xpm_colors (xpm_cache[name]);
   array<SI> hotspot= xpm_hotspot (xpm_cache[name]);
-  if (N(hotspot) == 0)
-    fatal_error ("Missing hotspot", "x_gui_rep::set_pointer");
+  ASSERT (N(hotspot) != 0, "missing hotspot");
   array<string> cnames_mask= xpm_colors (xpm_cache[mask_name]);
   char* bgcolor= as_charp (N(cnames_mask)>1 ? cnames_mask[1] :
 					      string ("white"));
@@ -469,21 +473,17 @@ x_gui_rep::set_mouse_pointer (widget w, string name, string mask_name) {
   XColor exact1, closest1;
   XLookupColor(dpy, cols, fgcolor, &exact1, &closest1);
   if (XAllocColor (dpy, cols, &exact1)) fg= &exact1;
-  else
-  if (XAllocColor (dpy, cols, &closest1)) fg= &closest1;
-  else
-    fatal_error ("Unable to allocate fgcolor", "x_gui_rep::set_pointer");
+  else if (XAllocColor (dpy, cols, &closest1)) fg= &closest1;
+  else FAILED ("unable to allocate fgcolor");
 
   XColor exact2, closest2;
   XLookupColor(dpy, cols, bgcolor, &exact2, &closest2);
   if (XAllocColor (dpy, cols, &exact2)) bg= &exact2;
-  else
-  if (XAllocColor (dpy, cols, &closest2)) bg= &closest2;
-  else
-    fatal_error ("Unable to allocate bgcolor", "x_gui_rep::set_pointer");
+  else if (XAllocColor (dpy, cols, &closest2)) bg= &closest2;
+  else FAILED ("unable to allocate bgcolor");
 
-  delete[] bgcolor;
-  delete[] fgcolor;
+  tm_delete_array (bgcolor);
+  tm_delete_array (fgcolor);
 
   SI x= hotspot[0], y= hotspot[1];
   Cursor cursor=XCreatePixmapCursor (dpy, curs, mask, fg, bg, x, y);
@@ -518,7 +518,7 @@ x_gui_rep::unmap_balloon () {
   if (!is_nil (balloon_wid)) {
     if (balloon_win != NULL) {
       balloon_win->set_visibility (false);
-      delete balloon_win;
+      tm_delete (balloon_win);
       balloon_win= NULL;
     }
     balloon_wid= widget ();
