@@ -4,10 +4,9 @@
 * DESCRIPTION: TeXmacs links by pipes
 * COPYRIGHT  : (C) 2000  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "pipe_link.hpp"
@@ -57,7 +56,7 @@ pipe_link_rep::~pipe_link_rep () {
 
 tm_link
 make_pipe_link (string cmd) {
-  return new pipe_link_rep (cmd);
+  return tm_new<pipe_link_rep> (cmd);
 }
 
 /******************************************************************************
@@ -75,7 +74,7 @@ execute_shell (string s) {
   argv[2] = _s;
   argv[3] = NULL;
   execve ("/bin/sh", argv, environ);
-  delete[] _s;
+  tm_delete_array (_s);
 }
 #endif
 #endif
@@ -91,13 +90,13 @@ pipe_link_rep::start () {
   bool success;
   cmdString = as_charp(cmd);
   success = PIPE_Create(cmdString, &conn);
-  delete [] cmdString;
+  tm_delete_array (cmdString);
   if (!success) return "Error: Could not create pipe";
   else {
 #else
-  pipe (pp_in );
-  pipe (pp_out);
-  pipe (pp_err);
+  int e1= pipe (pp_in ); (void) e1;
+  int e2= pipe (pp_out); (void) e2;
+  int e3= pipe (pp_err); (void) e3;
   pid= fork ();
   if (pid==0) { // the child
     setsid();
@@ -181,9 +180,10 @@ pipe_link_rep::write (string s, int channel) {
 #ifdef OS_WIN32
   PIPE_WriteStdin(&conn, _s, N(s));
 #else
-  ::write (in, _s, N(s));
+  int err= ::write (in, _s, N(s));
+  (void) err;
 #endif
-  delete[] _s;
+  tm_delete_array (_s);
 #endif
 }
 
