@@ -4,10 +4,9 @@
 * DESCRIPTION: Buffer management for TeXmacs server
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "tm_data.hpp"
@@ -86,9 +85,10 @@ tm_data_rep::new_menu_name (url u) {
 static void
 menu_append_buffer (string& s, tm_buffer buf) {
   if (buf->in_menu) {
-    s << " (\"" << buf->abbr;
-    if (buf->needs_to_be_saved ()) s << " *"; 
-    s << "\" (switch-to-buffer \"" * as_string (buf->name) * "\"))";
+    string name= copy (buf->abbr);
+    if (buf->needs_to_be_saved ()) name << " *"; 
+    s << " (" << scm_quote (name);
+    s << " (switch-to-buffer " * scm_quote (as_string (buf->name)) * "))";
   }
 }
 
@@ -165,7 +165,7 @@ tm_buffer
 tm_data_rep::new_buffer (url name) {
   int nr= find_buffer (name);
   if (nr != -1) return bufs[nr];
-  tm_buffer buf= new tm_buffer_rep (name);
+  tm_buffer buf= tm_new<tm_buffer_rep> (name);
   buf->abbr= new_menu_name (name);
   menu_insert_buffer (buf);
   return buf;
@@ -204,7 +204,7 @@ tm_data_rep::revert_buffer (url name, tree doc) {
   if (N(buf->vws)==0) set_document (buf->rp, extract (doc, "body"));
   else for (i=0; i<N(buf->vws); i++) {
     tm_view vw= buf->vws[i];
-    if (i==0) vw->ed->assign (vw->ed->rp, extract (doc, "body"));
+    if (i==0) assign (vw->ed->rp, extract (doc, "body"));
     vw->ed->set_style (buf->style);
     vw->ed->set_init  (buf->init);
     vw->ed->set_fin   (buf->fin);
@@ -224,7 +224,7 @@ tm_data_rep::delete_buffer (tm_buffer buf) {
   menu_delete_buffer (buf);
   for (i=0; i<N(buf->vws); i++)
     delete_view (buf->vws[i]);
-  delete buf;
+  tm_delete (buf);
 }
 
 void
@@ -286,7 +286,7 @@ tm_data_rep::new_view (url name) {
 
   tm_buffer buf= new_buffer (name);
   editor    ed = new_editor (get_server (), buf);
-  tm_view   vw = new tm_view_rep (buf, ed);
+  tm_view   vw = tm_new<tm_view_rep> (buf, ed);
   buf->vws << vw;
 
   ed->set_style (buf->style);
@@ -332,7 +332,9 @@ tm_data_rep::delete_view (tm_view vw) {
 	else a[j]= buf->vws[j+1];
       buf->vws= a;
     }
-  delete vw;
+  // tm_delete (vw);
+  // FIXME: causes very annoying segfault;
+  // recently introduced during reorganization
 }
 
 void
@@ -341,8 +343,7 @@ tm_data_rep::attach_view (tm_window win, tm_view vw) {
   vw->win= win;
   widget wid= win->wid;
   set_canvas (wid, vw->ed);
-  if (!is_attached (wid))
-    fatal_error ("widget should be attached", "tm_data_rep::attach_view");
+  ASSERT (is_attached (wid), "widget should be attached");
   vw->ed->resume ();
   win->set_window_name (vw->buf->abbr);
   // cout << "View attached\n";
@@ -355,8 +356,7 @@ tm_data_rep::detach_view (tm_view vw) {
   if (win == NULL) return;
   vw->win= NULL;
   widget wid= win->wid;
-  if (!is_attached (wid))
-    fatal_error ("widget should be attached", "tm_data_rep::attach_view");
+  ASSERT (is_attached (wid), "widget should be attached");
   vw->ed->suspend ();
   set_canvas (wid, glue_widget ());
   win->set_window_name ("TeXmacs");
@@ -382,8 +382,8 @@ tm_data_rep::new_window (bool map_flag, tree geom) {
   if (get_preference ("context dependent icons") == "on") mask += 4;
   if (get_preference ("user provided icons") == "on") mask += 8;
   if (get_preference ("status bar") == "on") mask += 16;
-  command quit= new kill_window_command_rep ();
-  tm_window win= new tm_window_rep (texmacs_widget (mask, quit), geom);
+  command quit= tm_new<kill_window_command_rep> ();
+  tm_window win= tm_new<tm_window_rep> (texmacs_widget (mask, quit), geom);
   if (map_flag) win->map ();
   return win;
 }
@@ -410,7 +410,7 @@ tm_data_rep::delete_window (tm_window win) {
   while (delete_view_from_window (win));
   win->unmap ();
   destroy_window_widget (win->win);
-  delete win;
+  tm_delete (win);
 }
 
 /******************************************************************************
@@ -544,11 +544,9 @@ tm_data_rep::kill_buffer () {
   if (N(bufs) <= 1) quit();
   tm_buffer buf= get_buffer();
   for (nr=0; nr<N(bufs); nr++) if (buf == bufs[nr]) break;
-  if (nr == N(bufs))
-    fatal_error ("Buffer not found", "tm_data_rep::kill_buffer");
+  ASSERT (nr != N(bufs), "buffer not found");
   for (nr=0; nr<N(bufs); nr++) if (buf != bufs[nr]) break;
-  if (nr == N(bufs))
-    fatal_error ("No suitable new buffer", "tm_data_rep::kill_buffer");
+  ASSERT (nr != N(bufs), "no suitable new buffer");
   tm_buffer new_buf = bufs[nr];
 
   for (i=0; i<N(buf->vws); i++) {
@@ -704,7 +702,7 @@ tm_data_rep::set_buffer_tree (url name, tree doc) {
   if (nr == -1) new_buffer (name, tree (DOCUMENT));
   nr= find_buffer (name);
   tm_buffer buf= bufs[nr];
-  get_editor () -> assign (buf->rp, doc);
+  assign (buf->rp, doc);
 }
 
 tree
@@ -747,8 +745,8 @@ tm_data_rep::get_project_buffer_menu () {
   tm_buffer buf= get_buffer ();
   if (buf->prj == NULL) return eval ("(menu-dynamic)");
   string s ("(menu-dynamic ");
-  s << "(\"" << buf->prj->abbr << "\" ";
-  s << "(switch-to-buffer \"" * as_string (buf->prj->name) * "\"))";
+  s << "(" << scm_quote (buf->prj->abbr) << " ";
+  s << "(switch-to-buffer " * scm_quote (as_string (buf->prj->name)) * "))";
 
   tree t= subtree (the_et, buf->prj->rp);
   int i, j, n= N(t);
@@ -757,8 +755,8 @@ tm_data_rep::get_project_buffer_menu () {
       string name= as_string (head (buf->prj->name) * as_string (t[i][0]));
       for (j=N(name)-1; j>=0; j--)
 	if (name[j]=='/') break;
-      s << " (\"" << name (j+1, N(name)) << "\" ";
-      s << "(switch-to-buffer \"" * name * "\"))";
+      s << " (" << scm_quote (name (j+1, N(name))) << " ";
+      s << "(switch-to-buffer " * scm_quote (name) * "))";
     }
 
   s << ")";
@@ -775,7 +773,7 @@ path
 new_document () {
   int i, n= N(the_et);
   for (i=0; i<n; i++)
-    if (the_et[i] == UNINIT && nil (the_et[i]->obs)) {
+    if (the_et[i] == UNINIT) {
       assign (the_et[i], tree (DOCUMENT, ""));
       return path (i); // obtain_ip (the_et[i]);
     }
@@ -789,6 +787,7 @@ new_document () {
 void
 delete_document (path rp) {
   assign (subtree (the_et, rp), UNINIT);
+  clean_observers (subtree (the_et, rp));
 #ifdef EXPERIMENTAL
   global_notify_assign (rp, UNINIT);
 #endif
@@ -817,15 +816,15 @@ create_window_id () {
 }
 
 static path
-remove (path p, int i) {
-  if (nil (p)) return p;
+reset (path p, int i) {
+  if (is_nil (p)) return p;
   else if (p->item == i) return p->next;
-  else return path (p->item, remove (p->next, i));
+  else return path (p->item, reset (p->next, i));
 }
 
 void
 destroy_window_id (int i) {
-  the_windows= remove (the_windows, i);
+  the_windows= reset (the_windows, i);
 }
 
 int
